// src/scorer.ts
function isMatch(subject, query_lw, query_up) {
  let i = -1;
  let j = -1;
  const m = subject.length;
  const n = query_lw.length;
  if (!m || n > m) {
    return false;
  }
  while (++j < n) {
    const qj_lw = query_lw.charCodeAt(j);
    const qj_up = query_up.charCodeAt(j);
    while (++i < m) {
      const si = subject.charCodeAt(i);
      if (si === qj_lw || si === qj_up) {
        break;
      }
    }
    if (i === m) {
      return false;
    }
  }
  return true;
}
function computeScore(subject, subject_lw, preparedQuery) {
  const query = preparedQuery.query;
  const query_lw = preparedQuery.query_lw;
  const m = subject.length;
  const n = query.length;
  const acro = scoreAcronyms(subject, subject_lw, query, query_lw);
  const acro_score = acro.score;
  if (acro.count === n) {
    return scoreExact(n, m, acro_score, acro.pos);
  }
  const pos = subject_lw.indexOf(query_lw);
  if (pos > -1) {
    return scoreExactMatch(subject, subject_lw, query, query_lw, pos, n, m);
  }
  const score_row = new Array(n);
  const csc_row = new Array(n);
  const sz = scoreSize(n, m);
  const miss_budget = Math.ceil(miss_coeff * n) + 5;
  let miss_left = miss_budget;
  let csc_should_rebuild = true;
  for (let j = 0;j < n; j++) {
    score_row[j] = 0;
    csc_row[j] = 0;
  }
  let score = 0;
  for (let i = 0;i < m; i++) {
    const si_lw = subject_lw[i];
    if (!preparedQuery.charCodes[si_lw.charCodeAt(0)]) {
      if (csc_should_rebuild) {
        for (let j = 0;j < n; j++) {
          csc_row[j] = 0;
        }
        csc_should_rebuild = false;
      }
      continue;
    }
    score = 0;
    let score_diag = 0;
    let csc_diag = 0;
    let record_miss = true;
    csc_should_rebuild = true;
    for (let j = 0;j < n; j++) {
      const score_up = score_row[j];
      score = score > score_up ? score : score_up;
      let csc_score = 0;
      if (query_lw[j] === si_lw) {
        const start = isWordStart(i, subject, subject_lw);
        csc_score = csc_diag > 0 ? csc_diag : scoreConsecutives(subject, subject_lw, query, query_lw, i, j, start);
        const align = score_diag + scoreCharacter(i, j, start, acro_score, csc_score);
        if (align > score) {
          score = align;
          miss_left = miss_budget;
        } else {
          if (record_miss && --miss_left <= 0) {
            return Math.max(score, score_row[n - 1]) * sz;
          }
          record_miss = false;
        }
      }
      score_diag = score_up;
      csc_diag = csc_row[j];
      csc_row[j] = csc_score;
      score_row[j] = score;
    }
  }
  score = score_row[n - 1];
  return score * sz;
}
function isWordStart(pos, subject, subject_lw) {
  if (pos === 0)
    return true;
  const curr_s = subject[pos];
  const prev_s = subject[pos - 1];
  return isSeparator(prev_s) || curr_s !== subject_lw[pos] && prev_s === subject_lw[pos - 1];
}
function isSeparator(c) {
  return c === " " || c === "." || c === "-" || c === "_" || c === "/" || c === "\\";
}
function scorePosition(pos) {
  if (pos < pos_bonus) {
    const sc = pos_bonus - pos;
    return 100 + sc * sc;
  } else {
    return Math.max(100 + pos_bonus - pos, 0);
  }
}
function scoreSize(n, m) {
  return tau_size / (tau_size + Math.abs(m - n));
}
function scoreExact(n, m, quality, pos) {
  return 2 * n * (wm * quality + scorePosition(pos)) * scoreSize(n, m);
}
function scorePattern(count, len, sameCase, start, end) {
  let sz = count;
  let bonus = 6;
  if (sameCase === count)
    bonus += 2;
  if (start)
    bonus += 3;
  if (end)
    bonus += 1;
  if (count === len) {
    if (start) {
      if (sameCase === len) {
        sz += 2;
      } else {
        sz += 1;
      }
    }
    if (end) {
      bonus += 1;
    }
  }
  return sameCase + sz * (sz + bonus);
}
function scoreCharacter(i, j, start, acro_score, csc_score) {
  const posBonus = scorePosition(i);
  if (start) {
    return posBonus + wm * ((acro_score > csc_score ? acro_score : csc_score) + 10);
  }
  return posBonus + wm * csc_score;
}
function scoreConsecutives(subject, subject_lw, query, query_lw, i, j, startOfWord) {
  let m = subject.length;
  let n = query.length;
  let mi = m - i;
  let nj = n - j;
  let k = mi < nj ? mi : nj;
  let sameCase = 0;
  let sz = 0;
  if (query[j] === subject[i]) {
    sameCase++;
  }
  while (++sz < k && query_lw[++j] === subject_lw[++i]) {
    if (query[j] === subject[i]) {
      sameCase++;
    }
  }
  if (sz < k) {
    i--;
  }
  if (sz === 1) {
    return 1 + 2 * sameCase;
  }
  return scorePattern(sz, n, sameCase, startOfWord, isWordEnd(i, subject, subject_lw, m));
}
function scoreExactMatch(subject, subject_lw, query, query_lw, pos, n, m) {
  let start = isWordStart(pos, subject, subject_lw);
  let sameCase = 0;
  if (!start) {
    let pos2 = subject_lw.indexOf(query_lw, pos + 1);
    if (pos2 > -1) {
      start = isWordStart(pos2, subject, subject_lw);
      if (start) {
        pos = pos2;
      }
    }
  }
  for (let i = 0;i < n; i++) {
    if (query[i] === subject[pos + i]) {
      sameCase++;
    }
  }
  let end = isWordEnd(pos + n - 1, subject, subject_lw, m);
  return scoreExact(n, m, scorePattern(n, n, sameCase, start, end), pos);
}
function isWordEnd(pos, subject, subject_lw, len) {
  if (pos === len - 1)
    return true;
  const curr_s = subject[pos];
  const next_s = subject[pos + 1];
  return isSeparator(next_s) || curr_s === subject_lw[pos] && next_s !== subject_lw[pos + 1];
}
function scoreAcronyms(subject, subject_lw, query, query_lw) {
  const m = subject.length;
  const n = query.length;
  if (!(m > 1 && n > 1)) {
    return emptyAcronymResult;
  }
  let count = 0;
  let sepCount = 0;
  let sumPos = 0;
  let sameCase = 0;
  let i = -1;
  let j = -1;
  while (++j < n) {
    const qj_lw = query_lw[j];
    if (isSeparator(qj_lw)) {
      i = subject_lw.indexOf(qj_lw, i + 1);
      if (i > -1) {
        sepCount++;
        continue;
      } else {
        break;
      }
    }
    while (++i < m) {
      if (qj_lw === subject_lw[i] && isWordStart(i, subject, subject_lw)) {
        if (query[j] === subject[i]) {
          sameCase++;
        }
        sumPos += i;
        count++;
        break;
      }
    }
    if (i === m) {
      break;
    }
  }
  if (count < 2) {
    return emptyAcronymResult;
  }
  const fullWord = count === n ? isAcronymFullWord(subject, subject_lw, query, count) : false;
  const score = scorePattern(count, n, sameCase, true, fullWord);
  return new AcronymResult(score, sumPos / count, count + sepCount);
}
function isAcronymFullWord(subject, subject_lw, query, nbAcronymInQuery) {
  const m = subject.length;
  const n = query.length;
  if (m > 12 * n)
    return false;
  let count = 0;
  for (let i = 0;i < m; i++) {
    if (isWordStart(i, subject, subject_lw) && ++count > nbAcronymInQuery) {
      return false;
    }
  }
  return true;
}
var wm = 150;
var pos_bonus = 20;
var tau_size = 150;
var miss_coeff = 0.75;
var scorer = {
  score(string, query, options) {
    const preparedQuery = options.preparedQuery || new Query(query, options);
    const allowErrors = options.allowErrors || false;
    if (!(allowErrors || isMatch(string, preparedQuery.core_lw, preparedQuery.core_up))) {
      return 0;
    }
    const string_lw = string.toLowerCase();
    const score = computeScore(string, string_lw, preparedQuery);
    return Math.ceil(score);
  }
};

class AcronymResult {
  score;
  pos;
  count;
  constructor(score, pos, count) {
    this.score = score;
    this.pos = pos;
    this.count = count;
  }
}
var emptyAcronymResult = new AcronymResult(0, 0.1, 0);

// src/pathScorer.ts
function calculateScorePath(subject, subject_lw, fullPathScore, options) {
  if (fullPathScore === 0) {
    return 0;
  }
  if (!options.preparedQuery)
    throw "IOptions.preparedQuery is required in calculateScorePath(), but it's missing.";
  const preparedQuery = options.preparedQuery;
  const useExtensionBonus = options.useExtensionBonus || false;
  const pathSeparator = options.pathSeparator || "/";
  let end = subject.length - 1;
  while (subject[end] === pathSeparator) {
    end--;
  }
  const basePos = subject.lastIndexOf(pathSeparator, end);
  const fileLength = end - basePos;
  let extAdjust = 1;
  if (useExtensionBonus) {
    extAdjust += getExtensionScore(subject_lw, preparedQuery.ext, basePos, end, 2);
    fullPathScore *= extAdjust;
  }
  if (basePos === -1) {
    return fullPathScore;
  }
  const depth = preparedQuery.depth;
  let basePos2 = basePos;
  let depth_penalty = 0;
  while (basePos2 > -1 && depth_penalty < depth) {
    depth_penalty++;
    basePos2 = subject.lastIndexOf(pathSeparator, basePos2 - 1);
  }
  const basePathScore = basePos === -1 ? fullPathScore : extAdjust * computeScore(subject.slice(basePos + 1, end + 1), subject_lw.slice(basePos + 1, end + 1), preparedQuery);
  const alpha = 0.5 * tau_depth / (tau_depth + countDir(subject, end + 1, pathSeparator));
  return alpha * basePathScore + (1 - alpha) * fullPathScore * scoreSize(depth_penalty, file_coeff * fileLength);
}
function countDir(path, end, pathSeparator) {
  if (end < 1) {
    return 0;
  }
  let count = 0;
  let i = -1;
  while (++i < end && path[i] === pathSeparator) {
    continue;
  }
  while (++i < end) {
    if (path[i] === pathSeparator) {
      count++;
      while (++i < end && path[i] === pathSeparator) {
        continue;
      }
    }
  }
  return count;
}
function getExtension(str) {
  const pos = str.lastIndexOf(".");
  return pos < 0 ? "" : str.substr(pos + 1);
}
function getExtensionScore(candidate, ext, startPos, endPos, maxDepth) {
  if (!ext.length) {
    return 0;
  }
  const pos = candidate.lastIndexOf(".", endPos);
  if (!(pos > startPos)) {
    return 0;
  }
  const ext_len = ext.length;
  const ext_pos = pos + 1;
  const len = endPos - ext_pos + 1;
  if (len < ext_len) {
    return 0.9 * len / ext_len;
  }
  const match_len = getCommonSubsequenceLength(candidate, ext, ext_pos, ext_len);
  if (match_len === 0 && maxDepth > 0) {
    return 0.9 * getExtensionScore(candidate, ext, startPos, pos - 1, maxDepth - 1);
  }
  return match_len / len;
}
function getCommonSubsequenceLength(a, b, startIndex, length) {
  let matched = 0;
  for (let i = 0;i < length; i++) {
    if (a[startIndex + i] === b[i]) {
      matched++;
    }
  }
  return matched;
}
var tau_depth = 20;
var file_coeff = 2.5;
var pathScorer = {
  score(string, query, options) {
    const preparedQuery = options.preparedQuery || new Query(query, options);
    const allowErrors = options.allowErrors || false;
    if (!(allowErrors || isMatch(string, preparedQuery.core_lw, preparedQuery.core_up))) {
      return 0;
    }
    const string_lw = string.toLowerCase();
    const score = computeScore(string, string_lw, preparedQuery);
    const scorePath = calculateScorePath(string, string_lw, score, options);
    return Math.ceil(scorePath);
  }
};

// src/filter.ts
function filterInternal(candidates, query, options) {
  const scoredCandidates = [];
  const { key, maxResults, maxInners, usePathScoring } = options;
  let spotLeft = maxInners != null && maxInners > 0 ? maxInners : candidates.length + 1;
  const bKey = key != null;
  const scoreProvider = usePathScoring ? pathScorer : scorer;
  for (let candidate of candidates) {
    const string = bKey ? candidate[key] : candidate;
    if (!string) {
      continue;
    }
    const score = scoreProvider.score(string, query, options);
    if (score > 0) {
      scoredCandidates.push({ candidate, score });
      if (!--spotLeft) {
        break;
      }
    }
  }
  scoredCandidates.sort((a, b) => b.score - a.score);
  let result = scoredCandidates.map(({ candidate }) => candidate);
  if (maxResults != null) {
    result = result.slice(0, maxResults);
  }
  return result;
}

// src/matcher.ts
var match = (string, query, options) => {
  const { allowErrors, preparedQuery, pathSeparator } = options;
  if (!(allowErrors || isMatch(string, preparedQuery.core_lw, preparedQuery.core_up))) {
    return [];
  }
  const string_lw = string.toLowerCase();
  let matches = computeMatch(string, string_lw, preparedQuery);
  if (matches.length === 0) {
    return matches;
  }
  if (string.indexOf(pathSeparator) > -1) {
    const baseMatches = basenameMatch(string, string_lw, preparedQuery, pathSeparator);
    matches = mergeMatches(matches, baseMatches);
  }
  return matches;
};
var wrap = (string, query, options) => {
  const { wrap: wrapOptions = {} } = options;
  let { tagClass = "highlight", tagOpen, tagClose } = wrapOptions;
  if (!tagOpen) {
    tagOpen = `<strong class="${tagClass}">`;
  }
  if (!tagClose) {
    tagClose = "</strong>";
  }
  if (string === query) {
    return tagOpen + string + tagClose;
  }
  const matchPositions = match(string, query, options);
  if (matchPositions.length === 0) {
    return string;
  }
  let output = "";
  let matchIndex = -1;
  let strPos = 0;
  while (++matchIndex < matchPositions.length) {
    let matchPos = matchPositions[matchIndex];
    if (matchPos > strPos) {
      output += string.substring(strPos, matchPos);
      strPos = matchPos;
    }
    while (++matchIndex < matchPositions.length) {
      if (matchPositions[matchIndex] === matchPos + 1) {
        matchPos++;
      } else {
        matchIndex--;
        break;
      }
    }
    matchPos++;
    if (matchPos > strPos) {
      output += tagOpen;
      output += string.substring(strPos, matchPos);
      output += tagClose;
      strPos = matchPos;
    }
  }
  if (strPos <= string.length - 1) {
    output += string.substring(strPos);
  }
  return output;
};
var basenameMatch = (subject, subject_lw, preparedQuery, pathSeparator) => {
  let end = subject.length - 1;
  while (subject[end] === pathSeparator) {
    end--;
  }
  let basePos = subject.lastIndexOf(pathSeparator, end);
  if (basePos === -1) {
    return [];
  }
  let depth = preparedQuery.depth;
  while (depth-- > 0) {
    basePos = subject.lastIndexOf(pathSeparator, basePos - 1);
    if (basePos === -1) {
      return [];
    }
  }
  basePos++;
  end++;
  return computeMatch(subject.slice(basePos, end), subject_lw.slice(basePos, end), preparedQuery, basePos);
};
var mergeMatches = (a, b) => {
  const m = a.length;
  const n = b.length;
  if (n === 0) {
    return a.slice();
  }
  if (m === 0) {
    return b.slice();
  }
  const out = [];
  let i = 0, j = 0;
  let bj = b[j];
  while (i < m) {
    const ai = a[i];
    while (bj <= ai && ++j < n) {
      if (bj < ai) {
        out.push(bj);
      }
      bj = b[j];
    }
    out.push(ai);
    i++;
  }
  while (j < n) {
    out.push(b[j++]);
  }
  return out;
};
var computeMatch = (subject, subject_lw, preparedQuery, offset = 0) => {
  const query = preparedQuery.query;
  const query_lw = preparedQuery.query_lw;
  const m = subject.length;
  const n = query.length;
  const acro = scoreAcronyms(subject, subject_lw, query, query_lw);
  const acro_score = acro.score;
  if (acro.count === n) {
    return scoreExact2(n, m, acro_score, acro.pos);
  }
  const pos = subject_lw.indexOf(query_lw);
  if (pos > -1) {
    return scoreExactMatch2(subject, subject_lw, query, query_lw, pos, n, m);
  }
  const score_row = new Array(n).fill(0);
  const csc_row = new Array(n).fill(0);
  const sz = scoreSize2(n, m);
  const miss_budget = Math.ceil(0.75 * n) + 5;
  let miss_left = miss_budget;
  let csc_should_rebuild = true;
  const matches = [];
  for (let i = 0;i < m; i++) {
    const si_lw = subject_lw[i];
    if (!preparedQuery.charCodes[si_lw.charCodeAt(0)]) {
      if (csc_should_rebuild) {
        for (let j = 0;j < n; j++) {
          csc_row[j] = 0;
        }
        csc_should_rebuild = false;
      }
      continue;
    }
    let score = 0;
    let score_diag = 0;
    let csc_diag = 0;
    let record_miss = true;
    csc_should_rebuild = true;
    for (let j = 0;j < n; j++) {
      const score_up = score_row[j];
      if (score_up > score) {
        score = score_up;
      }
      let csc_score = 0;
      if (query_lw[j] === si_lw) {
        const start = isWordStart(i, subject, subject_lw);
        csc_score = csc_diag > 0 ? csc_diag : scoreConsecutives(subject, subject_lw, query, query_lw, i, j, start);
        const align = score_diag + scoreCharacter(i, j, start, acro_score, csc_score);
        if (align > score) {
          score = align;
          miss_left = miss_budget;
        } else {
          if (record_miss && --miss_left <= 0) {
            return matches;
          }
          record_miss = false;
        }
        matches.push(i + offset);
      }
      score_diag = score_up;
      csc_diag = csc_row[j];
      csc_row[j] = csc_score;
      score_row[j] = score;
    }
  }
  return matches;
};
var scoreSize2 = (n, m) => {
  const tau_size2 = 150;
  return tau_size2 / (tau_size2 + Math.abs(m - n));
};
var scoreExact2 = (n, m, quality, pos) => {
  const wm2 = 150;
  const sz = scoreSize2(n, m);
  return Array.from({ length: n }, (_, i) => i + pos);
};
var scoreExactMatch2 = (subject, subject_lw, query, query_lw, pos, n, m) => {
  let start = isWordStart(pos, subject, subject_lw);
  if (!start) {
    const pos2 = subject_lw.indexOf(query_lw, pos + 1);
    if (pos2 > -1) {
      start = isWordStart(pos2, subject, subject_lw);
      if (start) {
        pos = pos2;
      }
    }
  }
  let sameCase = 0;
  for (let i = 0;i < n; i++) {
    if (query[i] === subject[pos + i]) {
      sameCase++;
    }
  }
  const end = isWordEnd2(pos + n - 1, subject, subject_lw, m);
  return Array.from({ length: n }, (_, i) => i + pos);
};
var isWordEnd2 = (pos, subject, subject_lw, len) => {
  if (pos === len - 1) {
    return true;
  }
  const curr_s = subject[pos];
  const next_s = subject[pos + 1];
  return isSeparator2(next_s) || curr_s === subject_lw[pos] && next_s !== subject_lw[pos + 1];
};
var isSeparator2 = (c) => {
  return c === " " || c === "." || c === "-" || c === "_" || c === "/" || c === "\\";
};

// src/query.ts
function coreChars(query, optCharRegEx = opt_char_re) {
  return query.replace(optCharRegEx, "");
}
function truncatedUpperCase(str) {
  return str.split("").map((char) => char.toUpperCase()[0]).join("");
}
function getCharCodes(str) {
  const charCodes = [];
  for (let i = 0;i < str.length; i++) {
    charCodes[str.charCodeAt(i)] = true;
  }
  return charCodes;
}

class Query {
  $$__internal = Symbol("Query");
  query;
  query_lw;
  core;
  core_lw;
  core_up;
  depth;
  ext;
  charCodes;
  constructor(query, options = {}) {
    if (!query?.length)
      throw "query string can't be empty";
    this.query = query;
    this.query_lw = query.toLowerCase();
    this.core = coreChars(query, options.optCharRegEx);
    this.core_lw = this.core.toLowerCase();
    this.core_up = truncatedUpperCase(this.core);
    this.depth = countDir(query, query.length, options.pathSeparator);
    this.ext = getExtension(this.query_lw);
    this.charCodes = getCharCodes(this.query_lw);
  }
}
var opt_char_re = /[ _\-:\/\\]/g;

// src/index.ts
function filter2(candidates, query2, options) {
  return filterInternal(candidates, query2, options || {});
}
function score(string, query2, options) {
  return scorer.score(string, query2, options || {});
}
function match2(string, query2, options) {
  return match(string, query2, {
    allowErrors: options?.allowErrors || false,
    preparedQuery: options?.preparedQuery || new Query(query2, options),
    pathSeparator: options?.pathSeparator || "/"
  });
}
function prepareQuery(query2, options) {
  return new Query(query2, options);
}
export {
  wrap,
  score,
  prepareQuery,
  match2 as match,
  filter2 as filter,
  Query
};

//# debugId=C2B2F126FF8F4F8364756E2164756E21
//# sourceMappingURL=index.js.map
